@@ -32,10 +32,6 @@
 #include <iphlpapi.h>
 #include <shlobj.h>
 
-#ifdef _MSC_VER
-#pragma comment(lib, "IPHLPAPI.lib")
-#endif // _MSC_VER
-
 #define MALLOC(x) HeapAlloc(GetProcessHeap(), 0, (x))
 #define FREE(x) HeapFree(GetProcessHeap(), 0, (x))
 
@@ -521,7 +517,7 @@ namespace net
 
 	bool IsLocalAddress (const boost::asio::ip::address& addr)
 	{
-		auto mtu =  // TODO: implement better
+		auto mtu = // TODO: implement better
 #ifdef _WIN32
 		GetMTUWindows(addr, 0);
 #else
